import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Write a description of class Vector here.
 * 
 * @author (Anthony Zepperi)
 * @version (1.11)
 */
public final class Vector
{
    double dx;
    double dy;
    int direction;
    double length;
    
    /**
     * Create a new. neutral vector.
     */
    public Vector()
    {
    }
    
   
    public Vector(int direction, double length)
    {
        this.length = length;
        this.direction = direction;
        updateCartesian();
    }
    
   
    public Vector(double dx, double dy)
    {
        this.dx = dx;
        this.dy = dy;
        updatePolar();
    }
    
    /**
     * Set the direction of this vector, leaving the length intact.
     */
    public void setDirection(int direction)
    {
        this.direction = direction;
        updateCartesian();
    }
    
    /**
     * Add another vector to this vector.
     */
    public void add(Vector other)
    {
        dx += other.dx;
        dy += other.dy;
        updatePolar();
    }
    
    public void setLength(double length)
    {
        this.length = length;
        updateCartesian();
    }    
    public void subtract(Vector other)
    {
        dx -= other.dx;
        dy -= other.dy;
        updatePolar();
    }
    /**
     * Reduce the length of this vector, leaving the direction intact/
     */
    public void reduceLength(double d)
    {
        length = length - d;
        updateCartesian();
    }
   
    public void scale(double factor)
    {
        length = length * factor;
        updateCartesian();
    }
    
    /**
     * Set this vector to the natural vector (length 0).
     */
    public void setNeutral() {
        dx = 0.0;
        dy = 0.0;
        length = 0.0;
        direction = 0;
    }
    
    /** 
     * Revert to horizontal component of this movement vector.
     */
    public void revertHorizontal() {
        dx = -dx;
        updatePolar();
    }
    
    /**
     * Revert to vertical component of this movement vector.
     */
    public void revertVertical() {
        dy = -dy;
        updatePolar();
    }    
    
    /**
     * Return the x offset of this vector (start to end points).
     */
    public double getX() {
        return dx;
    }
    
    /**
     * Return the y offset of this vector (start to end point).
     */
    public double getY() {
        return dy;
    }
    
    /**
     * Return the direction of this vector (in degrees).  0 is EAST.
     */
    public int getDirtection() {
        return direction;
    }
    
    /**
     * Return the length of this vector.
     */
    public double getLength() {
        return length;
    }
    
    /**
     * Update the direction and length from the current dx, dy.
     */
    private void updatePolar()
    {
        this.direction = (int) Math.toDegrees(Math.atan2(dy, dx));
        this.length = Math.sqrt(dx*dx+dy*dy);
    }
    
    /**
     * Update dx and dy from the current direction and length.
     */
    private void updateCartesian()
    {
        dx = length * Math.cos(Math.toRadians(direction));
        dy = length * Math.sin(Math.toRadians(direction));
    }
}
